from abc import ABC
from urllib.parse import urlencode

from requests import request
from requests_toolbelt.multipart.encoder import MultipartEncoder

from asc.core import log


class APIRequestor(ABC):

    base_api_url:str  # Não informar '/' ao final.

    base_queries:dict = {}
    base_payload:dict = {}
    base_headers:dict = {}

    data_payload = {}
    data_payload_as_multipart = False

    @property
    def payload_key(self) -> str:
        """ Chave a ser utilizada ao enviar payload no request. """

        if self.data_payload_as_multipart:
            return 'data'

        return 'json'

    def read_response(self, response, parse_as:str = 'json'):
        if parse_as == 'json':
            resp = response.json()

            return resp

        return response.text

    def mount_url(self,url : str):
        URL = '%s/%s' % (self.base_api_url,url)
        log.debug(f'url={URL}')
        return URL

    def mountHeaders(self,headers:dict={}):
        headers = {
            **{
                k:' '.join(v) if isinstance(v,list) else v \
                    for k,v in self.base_headers.items()
            },
            **headers
        }

        if self.data_payload_as_multipart and self.data_payload:
            headers = {
                **headers,
                'Content-Type': self.data_payload.content_type
            }

        return headers

    def mountPayload(self,payload:dict={}):
        
        req_payload = self.base_payload or {}

        if payload: 
            req_payload = {
                **req_payload,
                **payload
            }

        self.data_payload = req_payload

        if self.data_payload_as_multipart:
            self.data_payload = MultipartEncoder(fields=self.data_payload)

        return self.data_payload

    def toURLQueryString(self,query:dict={}):

        query = {
            **self.base_queries,
            **query
        }

        if query:
            return '?%s' % urlencode(query)

        return ''

    def GET(self, url, query:dict={}):

        request_url = self.mount_url(url + self.toURLQueryString(query))
        request_headers = self.mountHeaders()
        request_payload = self.mountPayload()

        return request(
            "GET", 
            url=request_url,
            headers=request_headers,
            data=request_payload,
            files=[]
        )

    def POST(self, url, data:dict={}):
        payload_data = self.mountPayload(data)

        return request(
            "POST", 
            url=self.mount_url(url),
            headers=self.mountHeaders(), 
            **{self.payload_key:payload_data},
        )

    def DELETE(self, url, data:dict={}):
        payload_data = self.mountPayload(data)

        return request(
            "POST",
            url=self.mount_url(url),
            headers=self.mountHeaders(),
            **{self.payload_key: payload_data},
        )


    def PUT(self, url, data:dict={}):
        payload_data = self.mountPayload(data)

        return request(
            "PUT",
            url=self.mount_url(url),
            headers=self.mountHeaders(),
            **{self.payload_key: payload_data},
        )
